<?php

namespace FiloBlu\ScratchAndWin\Block\Adminhtml\ScratchAndWin\Edit\Tab;

class ScratchAndWinDetails extends \Magento\Backend\Block\Widget\Grid\Extended implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * Image collection factory
     *
     * @var \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWinDetails\CollectionFactory
     */
    protected $scratchAndWinDetailsCollectionFactory;

    /**
     * Registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry;

    /**
     * Image factory
     *
     * @var \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsFactory
     */
    protected $scratchAndWinDetailsFactory;

    /**
     * constructor
     *
     * @param \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWinDetails\CollectionFactory $scratchAndWinDetailsCollectionFactory
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsFactory $scratchAndWinDetailsFactory
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param array $data
     */
    public function __construct(
        \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWinDetails\CollectionFactory $scratchAndWinDetailsCollectionFactory,
        \Magento\Framework\Registry $coreRegistry,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsFactory $scratchAndWinDetailsFactory,
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        array $data = []
    )
    {
        $this->scratchAndWinDetailsCollectionFactory = $scratchAndWinDetailsCollectionFactory;
        $this->coreRegistry         = $coreRegistry;
        $this->scratchAndWinDetailsFactory           = $scratchAndWinDetailsFactory;
        parent::__construct($context, $backendHelper, $data);
    }


    /**
     * Set grid params
     */
    public function _construct()
    {
        parent::_construct();
        $this->setId('scratchandwindetails_grid');
        $this->setDefaultSort('position');
        $this->setDefaultDir('ASC');
        $this->setUseAjax(true);
        if ($this->getScratchAndWin()->getId()) {
            $this->setDefaultFilter(['in_details'=>1]);
        }
    }

    /**
     * prepare the collection

     * @return $this
     */
    protected function _prepareCollection()
    {
        /** @var \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWinDetails\Collection $collection */
        $collection = $this->scratchAndWinDetailsCollectionFactory->create();

        if ($this->getScratchAndWin()->getId()) {
            $constraint = 'related.filoblu_scratchandwin_id='.$this->getScratchAndWin()->getId();
        } else {
            $constraint = 'related.filoblu_scratchandwin_id=0';
        }
        $collection->getSelect()->joinLeft(
            array('related' => $collection->getTable('filoblu_scratchandwin_relations')),
            'related.filoblu_scratchandwin_details_id=main_table.id AND '.$constraint,
            array('filoblu_scratchandwin_id')
        );

        $this->setCollection($collection);
        parent::_prepareCollection();
        return $this;
    }

    /**
     * @return $this
     */
    protected function _prepareMassaction()
    {
        return $this;
    }

    /**
     * @return $this
     */
    protected function _prepareColumns()
    {
        $this->addColumn(
            'in_details',
            [
                'header_css_class'  => 'a-center',
                'type'   => 'checkbox',
                'name'   => 'in_image',
                'values' => $this->_getSelectedDetails(),
                'align'  => 'center',
                'index'  => 'id'
            ]
        );
        $this->addColumn(
            'filoblu_scratchandwin_details_id',
            [
                'header' => __('ID'),
                'sortable' => true,
                'index' => 'id',
                'type' => 'number',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'promocode',
            [
                'header' => __('Promocode'),
                'index' => 'promocode',
                'header_css_class' => 'col-name',
                'column_css_class' => 'col-name'
            ]
        );


        $this->addColumn(
            'position',
            [
                'header' => __('Position'),
                'name'   => 'position',
                'width'  => 60,
                'type'   => 'number',
                'validate_class' => 'validate-number',
                'index' => 'position',
                'editable'  => true,
            ]
        );

        return $this;
    }


    /**
     * Retrieve selected Details

     * @return array
     */
    protected function _getSelectedDetails()
    {
        $details = $this->getScratchAndWinDetails();
        if (!is_array($details)) {
            $details = $this->getScratchAndWin()->getDetailsPosition();

            return array_keys($details);
        }



        return $details;
    }

    /**
     * Retrieve selected Details

     * @return array
     */
    public function getSelectedDetails()
    {
        $selected = $this->getScratchAndWin()->getDetailsPosition();
        if (!is_array($selected)) {
            $selected = [];
        } else {
            foreach ($selected as $key => $value) {
                $selected[$key] = ['position' => $value];
            }
        }
        return $selected;
    }


    /**
     * @param \FiloBlu\ScratchAndWin\Model\ScratchAndWin|\Magento\Framework\Object $item
     * @return string
     */
    public function getRowUrl($item)
    {
        return '#';
    }

    /**
     * get grid url
     *
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl(
            '*/*/detailsGrid',
            [
                'store_id' => $this->getScratchAndWin()->getId()
            ]
        );
    }

    /**
     * @return \FiloBlu\ScratchAndWin\Model\ScratchAndWin
     */
    public function getScratchAndWin()
    {
        return $this->coreRegistry->registry('filoblu_scratchandwin');
    }

    /**
     * @param \Magento\Backend\Block\Widget\Grid\Column $column
     * @return $this
     */
    protected function _addColumnFilterToCollection($column)
    {
        if ($column->getId() == 'in_details') {
            $detailsIds = $this->_getSelectedDetails();

            if (empty($detailsIds)) {
                $detailsIds = 0;
            }

            if ($column->getFilter()->getValue()) {
                $this->getCollection()->addFieldToFilter('main_table.id', ['in'=>$detailsIds]);
            } else {
                if ($detailsIds) {
                    $this->getCollection()->addFieldToFilter('main_table.id', ['nin'=>$detailsIds]);
                }
            }
        } else {
            parent::_addColumnFilterToCollection($column);
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getTabLabel()
    {
        return __('Details');
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return $this->getRequest()->getParam('store',0) != 0;
    }

    /**
     * @return string
     */
    public function getTabTitle()
    {
        return $this->getTabLabel();
    }

    /**
     * @return bool
     */
    public function canShowTab()
    {
        return $this->getRequest()->getParam('store',0) == 0;
    }

    /**
     * @return string
     */
    public function getTabUrl()
    {
        return $this->getUrl('filobluscratchandwinadmin/scratchandwin/details', ['_current' => true]);
    }

    /**
     * @return string
     */
    public function getTabClass()
    {
        return 'ajax only';
    }
}
