<?php
namespace FiloBlu\ScratchAndWin\Controller\Adminhtml\ScratchAndWin;

use Magento\Backend\App\Action;

class Save extends \Magento\Backend\App\Action
{

    protected $uploaderFactory;
    protected $imageModel;
    protected $jsHelper;
    protected $scratchAndWinModelFactory;
    protected $scratchAndWinTranslationsModelFactory;
    
    public function __construct(
        Action\Context $context,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWin\Image $imageModel,
        \Magento\Backend\Helper\Js $jsHelper,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinFactory $scratchAndWinModelFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinTranslationsFactory $scratchAndWinTranslationsModelFactory 
    )
    {
        $this->uploaderFactory = $uploaderFactory;
        $this->imageModel = $imageModel;
        $this->jsHelper = $jsHelper;
        $this->scratchAndWinModelFactory = $scratchAndWinModelFactory;
        $this->scratchAndWinTranslationsModelFactory = $scratchAndWinTranslationsModelFactory;
        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_ScratchAndWin::config');
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        if ($data) {

            $id = $this->getRequest()->getParam('id');
            $storeId = $this->getRequest()->getParam('store',0);
            $isSavingTranslation = false;

            if ($id && $storeId != 0) {
                $isSavingTranslation = true;
            }

            $model = $this->scratchAndWinModelFactory->create();

            $modelTranslations = $this->scratchAndWinTranslationsModelFactory->create();

            if ($id) {
                $model->load($id);

                if ($isSavingTranslation) {
                    $modelTranslations->loadByParentAndStoreId($id,$storeId);
                }
            }

            $model->setData($data);

            $bgImage = $this->uploadFileAndGetName('bg_image', $this->imageModel->getBaseDir(), $data);
            $model->setBgImage($bgImage);

            $bgImageMobile = $this->uploadFileAndGetName('bg_image_mobile', $this->imageModel->getBaseDir(), $data);
            $model->setBgImageMobile($bgImageMobile);

            $bgVideo = $this->uploadFileAndGetName('bg_video', $this->imageModel->getBaseDir(), $data);
            $model->setBgVideo($bgVideo);

            try {

                if ($isSavingTranslation) {

                    $modelTranslations->setSerializedValues(serialize($model->getData()));
                    $modelTranslations->setFilobluScratchandwinId($id);
                    $modelTranslations->setStoreId($storeId);
                    $modelTranslations->save();

                }
                else {

                    $details = $this->getRequest()->getPost('details', -1);

                    if ($details != -1) {
                        $model->setDetailsData($this->jsHelper->decodeGridSerializedInput($details));
                    }

                    $this->_eventManager->dispatch(
                        'filoblu_scratchandwin_prepare_save',
                        [
                            'scratchandwin' => $model,
                            'request' => $this->getRequest()
                        ]
                    );                    
                    
                    $model->save();                    
                }

                $this->messageManager->addSuccess(__('You saved this ScratchAndWin.'));

                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId() ? : $id, 'store' => $storeId, '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\RuntimeException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the ScratchAndWin.'));
            }

            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id'),'store' => $storeId]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    public function uploadFileAndGetName($input, $destinationFolder, $data)
    {
        try {
            if (isset($data[$input]['delete'])) {
                return '';
            } else {
                $uploader = $this->uploaderFactory->create(['fileId' => $input]);
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                $uploader->setAllowCreateFolders(true);
                $result = $uploader->save($destinationFolder);
                return $result['file'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() != \Magento\Framework\File\Uploader::TMP_NAME_EMPTY) {
                throw new FrameworkException($e->getMessage());
            } else {
                if (isset($data[$input]['value'])) {
                    return $data[$input]['value'];
                }
            }
        }
        return '';
    }
}
