<?php
namespace FiloBlu\ScratchAndWin\Model\ResourceModel;

class ScratchAndWin extends \Magento\Framework\Model\ResourceModel\Db\AbstractDb
{
    /**
     * Event Manager
     *
     * @var \Magento\Framework\Event\ManagerInterface
     */
    protected $eventManager;

    protected $scratchandwinRelationTable;

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('filoblu_scratchandwin', 'id');
    }

    public function __construct(
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Framework\Model\ResourceModel\Db\Context $context
    )
    {
        $this->eventManager = $eventManager;

        parent::__construct($context);

        $this->scratchandwinRelationTable = $this->getTable('filoblu_scratchandwin_relations');
    }

    protected function _afterSave(\Magento\Framework\Model\AbstractModel $object) {
        $this->saveDetailsRelation($object);

        return parent::_afterSave($object);
    }

    protected function saveDetailsRelation(\FiloBlu\ScratchAndWin\Model\ScratchAndWin $scratchAndWin) {
        $scratchAndWin->setIsChangedDetailsList(false);
        $id = $scratchAndWin->getId();

        $details = $scratchAndWin->getDetailsData();
        if ($details === null) {
            return $this;
        }
        $oldDetails = $scratchAndWin->getDetailsPosition();
        $insert = array_diff_key($details, $oldDetails);
        $delete = array_diff_key($oldDetails, $details);
        $update = array_intersect_key($details, $oldDetails);
        $_update = array();
        foreach ($update as $key => $settings) {
            if (isset($oldDetails[$key]) && $oldDetails[$key] != $settings['position']) {
                $_update[$key] = $settings;
            }
        }
        $update = $_update;
        $adapter = $this->getConnection();
        if (!empty($delete)) {
            $condition = ['filoblu_scratchandwin_details_id IN(?)' => array_keys($delete), 'filoblu_scratchandwin_id=?' => $id];
            $adapter->delete($this->scratchandwinRelationTable, $condition);
        }
        if (!empty($insert)) {
            $data = [];
            foreach ($insert as $detailsId => $position) {
                $data[] = [
                    'filoblu_scratchandwin_id' => (int) $id,
                    'filoblu_scratchandwin_details_id' => (int) $detailsId,
                    'position' => (int) $position['position']
                ];
            }
            $adapter->insertMultiple($this->scratchandwinRelationTable, $data);
        }
        if (!empty($update)) {
            foreach ($update as $detailsId => $position) {
                $where = ['filoblu_scratchandwin_id = ?' => (int) $id, 'filoblu_scratchandwin_details_id = ?' => (int) $detailsId];
                $bind = ['position' => (int) $position['position']];
                $adapter->update($this->scratchandwinRelationTable, $bind, $where);
            }
        }
        if (!empty($insert) || !empty($delete)) {
//            $imageIds = array_unique(array_merge(array_keys($insert), array_keys($delete)));
            $this->eventManager->dispatch(
                'filoblu_scratchandwin_change_details', [
                    'scratchandwin' => $scratchAndWin,
                    'details_ids_insert' => $insert,
                    'details_ids_delete' => $delete,
                    'details_ids_update' => $update,
                ]
            );
        }
        if (!empty($insert) || !empty($update) || !empty($delete)) {
            $scratchAndWin->setIsChangedDetailsList(true);
            $detailsIds = array_keys($insert + $delete + $update);
            $scratchAndWin->setAffectedDetailsIds($detailsIds);
        }

        return $this;
    }

    public function getDetailsPosition(\FiloBlu\ScratchAndWin\Model\ScratchAndWin $scratchAndWin) {
        $select = $this->getConnection()->select()->from(
            $this->scratchandwinRelationTable, ['filoblu_scratchandwin_details_id', 'position']
        )
            ->where(
                'filoblu_scratchandwin_id = :filoblu_scratchandwin_id'
            );
        $bind = ['filoblu_scratchandwin_id' => (int) $scratchAndWin->getId()];

        return $this->getConnection()->fetchPairs($select, $bind);
    }

}
